@@ -25,6 +25,7 @@ import pkg_resources
 import subprocess
 import psutil
 import hashlib
+import datetime
 
 import requests
 
@@ -43,7 +44,7 @@ logger = createLogger(__name__)
 
 TEST_DB = 'db_test.sqlite'
 
-def importSettings(settingsDict, settings):  
+def importSettings(settingsDict, settings):
     settingsDict.update({k: getattr(settings, k) for k in dir(settings)})
     testSettings = {
         "SERVER_MODE": DEVELOPMENT,
@@ -58,37 +59,37 @@ def importSettings(settingsDict, settings):
     }
     attachSettings(settingsDict)
     settingsDict.update(testSettings)
-   
+
 CHROME_VERSION_PATH = os.path.join("chromedriver.txt")
 
 def downloadChromedriver():
     needDownload = False
-  
+
     if platform.system() in ['Linux', 'Darwin']:
         if not os.path.exists("chromedriver"):
             needDownload = True
     else:
         if not os.path.exists("chromedriver.exe"):
             needDownload = True
-            
+
     chromeVersion = requests.get("https://chromedriver.storage.googleapis.com/LATEST_RELEASE").text
-    
+
     if os.path.exists(CHROME_VERSION_PATH):
         with open(CHROME_VERSION_PATH, "r") as f:
             currentVersion = f.read()
     else:
         currentVersion = ""
-        
+
     if currentVersion == chromeVersion:
         needDownload = False
     else:
         # 只要版本不一樣，就重新下載
         needDownload = True
-    
+
     if needDownload:
         with open(CHROME_VERSION_PATH, "w") as f:
             f.write(chromeVersion)
-            
+
         if platform.system() == 'Linux':
             driverZip = requests.get(
                 "https://chromedriver.storage.googleapis.com/%s/chromedriver_linux64.zip" % chromeVersion).content
@@ -98,25 +99,25 @@ def downloadChromedriver():
         else:
             driverZip = requests.get(
                 "https://chromedriver.storage.googleapis.com/%s/chromedriver_win32.zip" % chromeVersion).content
-            
+
         zipPath = os.path.join("chromedriver.zip")
-                   
+
         with open(zipPath, "wb") as f:
             f.write(driverZip)
-            
+
         with zipfile.ZipFile(zipPath, 'r') as zip:
             zip.extractall(".")
-            
+
         if platform.system() in ['Linux', 'Darwin']:
             os.system("chmod +x chromedriver")
-            
+
         os.remove("chromedriver.zip")
 
 class RemoteTestCaseMixin(object):
 
     TEST_URL = None
     TEST_SETTINGS = None
-    
+
     def runTest(self):
         try:
             remoteURL = reverse("remoteTest")
@@ -137,9 +138,9 @@ class RemoteTestCaseMixin(object):
             if [int(i) for i in IUNO_VERSION.split(".")] > [int(i) for i in version.split(".")]:
                 logger.warning("Iuno version is %s, but latest Iuno version is %s." % (IUNO_VERSION, version))
         logger.info(allData.get('result', None))
-        
+
         errorCount = 0
-        
+
         for testcase in allData.get('track', []):
             if testcase.get('error', []):
                 classname = testcase.get('classname', None)
@@ -156,10 +157,10 @@ ERROR [%ss]: %s (%s)
                     logger.info("Type: %s" % error.get("type", None))
                     logger.info(error.get("traceback", None))
                     errorCount += 1
-        
+
         # has error
         self.assertEqual(0, errorCount)
-        
+
 SCHEMA_FIX = '''
     def __enter__(self):
         # Some SQLite schema alterations need foreign key constraints to be
@@ -219,49 +220,49 @@ SCHEMA_ORIGIN_DJANGO_2_1 = '''
 '''
 
 def fixSchema(schemaPath):
-    import django 
+    import django
     try:
         with open(schemaPath, 'r') as f:
             content = f.read()
-            
+
         if not django.VERSION >= (2, 0):
             SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_1_11
-            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11 
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11
             SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_1_11
         else:
             SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_2_1
             SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_2_1
             SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_2_1
-        
+
         if hashlib.md5(content.encode()).hexdigest() == SCHEMA_ORIGIN_MD5:
             content = content.replace(SCHEMA_ORIGIN, SCHEMA_FIX)
-              
+
         with open(schemaPath, 'w') as f:
             f.write(content)
-            
+
         with open(schemaPath, 'r') as f:
             content = f.read()
     except Exception as e:
         pass
-        
+
 def revertSchema(schemaPath):
-    import django 
+    import django
     try:
         with open(schemaPath, 'r') as f:
             content = f.read()
-            
+
         if not django.VERSION >= (2, 0):
             SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_1_11
-            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11 
+            SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_1_11
             SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_1_11
         else:
             SCHEMA_FIX_MD5 = SCHEMA_FIX_MD5_2_1
             SCHEMA_ORIGIN = SCHEMA_ORIGIN_DJANGO_2_1
             SCHEMA_ORIGIN_MD5 = SCHEMA_ORIGIN_MD5_DJANGO_2_1
-            
+
         if hashlib.md5(content.encode()).hexdigest() == SCHEMA_FIX_MD5:
             content = content.replace(SCHEMA_FIX, SCHEMA_ORIGIN)
-            
+
         with open(schemaPath, 'w') as f:
             f.write(content)
     except Exception as e:
@@ -273,7 +274,7 @@ def kill(pid):
         cp.kill()
     p.kill()
 
-        
+
 class SeleniumMixin(object):
 
     INITIAL_DATA = None
@@ -294,7 +295,7 @@ class SeleniumMixin(object):
         if cls.schemaPath:
             fixSchema(cls.schemaPath)
         cls.pid = cls._runServer(cls)
-        
+
         # Chrome
         downloadChromedriver()
         options = ChromeOptions()
@@ -302,12 +303,12 @@ class SeleniumMixin(object):
             options.add_argument("--headless")
         options.add_argument('--disable-gpu')
         options.add_argument('--no-sandbox')
-        
+
         if platform.system() == 'Linux' or platform.system() == 'Darwin':
             cls.browser = webdriver.Chrome(executable_path="./chromedriver", chrome_options=options)
         else:
             cls.browser = webdriver.Chrome(executable_path="./chromedriver.exe", chrome_options=options)
-        
+
         cls.browser.implicitly_wait(20)
 
     @staticmethod
@@ -316,17 +317,17 @@ class SeleniumMixin(object):
             revertSchema(cls.schemaPath)
         cls.browser.quit()
         kill(cls.pid)
-      
+
     # 因為 TestCase 不會寄信，因此改用自己 runServer
     # https://docs.djangoproject.com/en/1.8/topics/testing/tools/#email-services
     @staticmethod
     def _runServer(cls):
         env = os.environ.copy()
         envPython = sys.executable
-        
+
         if os.path.exists(TEST_DB):
             os.remove(TEST_DB)
-        
+
         os.system('"%s" manage.py migrate --settings %s' % (envPython, cls.TEST_SETTINGS))
 
 
@@ -335,20 +336,21 @@ class SeleniumMixin(object):
         else:
             os.system('"%s" manage.py loaddata %s %s --settings %s' % (envPython, cls.INITIAL_DATA, cls.LOADDATA_WITH_PARAMETER, cls.TEST_SETTINGS))
 
-       
+
         if cls.TEST_SETTINGS:
+            print('hi')
             p = subprocess.Popen(
                 ('"%s" manage.py runserver %s --noreload --settings %s') % (
-                    envPython, cls.PORT, cls.TEST_SETTINGS), 
+                    envPython, cls.PORT, cls.TEST_SETTINGS),
                 env=env, shell=True, cwd=".")
         else:
             p = subprocess.Popen(
                 ('"%s" manage.py runserver %s --noreload') % (
-                    envPython, cls.PORT), 
+                    envPython, cls.PORT),
                 env=env, shell=True, cwd=".")
-       
+
         time.sleep(20)
-       
+
         i = 0
         while i < 30: # 每 10 秒確認一次，試 5 次
             try:
@@ -360,6 +362,19 @@ class SeleniumMixin(object):
             finally:
                 time.sleep(10)
                 i += 1
-           
+
         raise RuntimeError("Run server time out.")
-    
\ No newline at end of file
+
+    @staticmethod
+    def _getEmail(EMAIL_FILE_PATH, time):
+        content = 'get mail error'
+        for filename in os.listdir(EMAIL_FILE_PATH):
+            if not filename:
+                return 'no mail'
+            if filename.endswith(".log"):
+                if datetime.datetime.strptime("-".join(filename.split('-')[0:2]), "%Y%m%d-%H%M%S") >= datetime.datetime.strptime(time, "%Y%m%d-%H%M%S"):
+                    with open(os.path.join(EMAIL_FILE_PATH, filename)) as f:
+                        content = f.read()
+                        return content
+
+        return content
